#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min','max';
use Math::BigRat;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()
use Math::Geometry::Planar ();

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'PartN','xy_rotate_minus90','transform_to_horizontal';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
use Smart::Comments;


print "  % generated by tools/$FindBin::Script\n";
my $level = 10;

my $path = Math::PlanePath::CCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
# my $n_lo = PartN($level);
# my $n_hi = PartN($level+1);

sub transform {
  my ($x,$y) = @_;
  ($x,$y) = transform_to_horizontal($x,$y, $level);
  $x /= 2;
  $y /= 2;
  return ($x,$y);
}

# x,y         x2,y2
#     <-----
#  * --------> *
#  ^ \       / |
#  |   mx,my   |
#  |           v
sub xydxdy_triangle_neighbours {
  my ($x,$y, $dx,$dy) = @_;
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
  return ([$x+$dx,$y+$dy, -$dx,-$dy],    # reversal
          [$x+$rx,$y+$ry, -$rx,-$ry],    # start
          [$x+$dx,$y+$dy, $rx,$ry]);     # end
}

sub gpc_print {
  my ($gpc) = @_;
  my @planars = Math::Geometry::Planar::Gpc2Polygons($gpc);
  my $indent = '';
  foreach my $planar (@planars) {
    print $indent,"planar ";
    foreach my $polygon ($planar->polygons) {
      print "poly ";
      my $points = $polygon->[0];
      ### $points
      print join(' ',map {join(',',@$_)} @$points),"\n";
      if (@$polygon > 1) { print " hole"; }
    }
    $indent = ' and ';
  }
}
sub gpc_num_planars {
  my ($gpc) = @_;
  my @planars = Math::Geometry::Planar::Gpc2Polygons($gpc);
  return scalar(@planars);
}


my $count_polygons = 0;
my $count_vertices = 0;
my $seen = '';
$Printwrap = 2;
printwrap("\\fill");
$Printwrap += 2;

foreach my $n ($n_lo .. $n_hi-1) {
  next if vec($seen,$n,1);
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);

  my $gpc = xydxdy_triangle_to_gpc($x,$y, $dx,$dy);
  my @pending = (xydxdy_triangle_neighbours($x,$y, $dx,$dy));
  vec($seen,$n,1) = 1;

  while (my $elem = shift @pending) {
    my ($x,$y, $dx,$dy) = @$elem;
    my $n = $path->xyxy_to_n ($x,$y, $x+$dx,$y+$dy) // next;
    $n < $n_hi or next;
    next if vec($seen,$n,1);

    $gpc = Math::Geometry::Planar::GpcClip
      ("UNION", $gpc, xydxdy_triangle_to_gpc($x,$y, $dx,$dy));
    push @pending, xydxdy_triangle_neighbours($x,$y, $dx,$dy);
    vec($seen,$n,1) = 1;

    unless (gpc_num_planars($gpc) == 1) {
      gpc_print($gpc);
      die "Oops, multiple polygons";
    }
  }

  # gpc_print($gpc);

  my @planars = Math::Geometry::Planar::Gpc2Polygons($gpc);
  @planars == 1 or die "Oops, multiple planars";
  my $polygons = $planars[0]->polygons;
  @$polygons == 1 or die "Oops, hole in polygon";
  my $points = $polygons->[0];

  my $join = ' ';
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    ($x,$y) = transform($x,$y);
    printwrap("$join($x,$y)");
    $count_vertices++;
    $join = '--';
  }
  $count_polygons++;
}
printwrap(";\n");

# my $gx = $x_total / $count_centroid;
# my $gy = $y_total / $count_centroid;
# my ($Gx,$Gy) = G_xy($level);
#
# my ($X_axis_x,$X_axis_y) = ($gx + $x_hi, $gy + $y_hi);
# my ($Y_axis_x,$Y_axis_y) = ($gx + $y_hi, $gy - $x_hi);
#
# print "    \\coordinate (centroid) at ($gx,$gy);\n";
# print "    \\coordinate (X-axis) at ($X_axis_x,$X_axis_y);\n";
# print "    \\coordinate (Y-axis) at ($Y_axis_x,$Y_axis_y);\n";

print "  % total $count_polygons polygons with $count_vertices vertices\n";
print "  % end generated\n";
exit 0;


#                  -4,4
#      -5,7            -5,5
#  -6,6    -4,6    -4,6
#
