#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'transform_to_horizontal','PartN','PartSegments';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::CCurve->new;

my $level = 10;
if (@ARGV) { $level = shift @ARGV; }

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

my $n_start = PartN($level-1);
my $n_end = PartN($level+1);

my $segments = MyLineSegments->new;
foreach my $n ($n_start .. $n_end-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($x,$y) = transform($x,$y);
  ($dx,$dy) = transform($dx,$dy);
  $segments->add($x,$y, $x+$dx,$y+$dy);
}

$Printwrap = 2;
printwrap("\\draw");
$Printwrap = 4;
$segments->printwrap;
printwrap(";\n");

{
  my ($x,$y) = $path->n_to_xy(PartN($level));
  ($x,$y) = transform($x,$y);
  print "  \\node (part-start) at ($x,$y) [my dot];\n";
}

print "  % end generated\n";
exit 0;
