#!/usr/bin/perl -w

# Copyright 2015, 2016, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'xy_rotate_minus90','transform_to_horizontal','rect_corners_sort',
  '@dir4_to_dx', '@dir4_to_dy';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 10;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::CCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

sub Three_num_neighbours_of_xydxdy {
  my ($x,$y, $dx,$dy) = @_;
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);

  my @n_list = ($path->xyxy_to_n($x2,$y2, $x,$y),
                $path->xyxy_to_n($x+$rx,$y+$ry, $x,$y),
                $path->xyxy_to_n($x2,$y2, $x2+$rx,$y2+$ry));
  ### @n_list
  @n_list = grep { defined $_ && $_ < $n_hi } @n_list;
  ### neighbours: "$n is ".scalar(@n_list)
  return scalar(@n_list);
}

my ($enc_x,$enc_y);

# find 3-side enclosed triangle
my $skip = 0;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  next unless Three_num_neighbours_of_xydxdy($x,$y, $dx,$dy) == 3;
  if ($skip-- > 0) { next; }

  my ($x2,$y2) = ($x+$dx, $y+$dy);
  print "  % Three at N=$n  $x,$y to $x2,$y2\n";
  $enc_x = $x;
  $enc_y = $y;
  last;
}

my $enc_x_min = $enc_x - 2;
my $enc_x_max = $enc_x + 1;
my $enc_y_min = $enc_y - 2;
my $enc_y_max = $enc_y + 3;

sub in_range {
  my ($x,$y) = @_;
  return ($enc_x_min <= $x && $x <= $enc_x_max
          && $enc_y_min <= $y && $y <= $enc_y_max);
}


#          gap 
# x1,y1 *------>* x2,y2
#        \     /
#      gap\   /gap
#          \ /
#           *
#          x3,y3
sub triangle {
  my ($x1,$y1, $x2,$y2) = @_;
  my ($dx,$dy) = ($x2-$x1, $y2-$y1);
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
  my ($x3,$y3) = ($x1+$dx/2+$rx/2, $y1+$dy/2+$ry/2);

  if (in_range($x1,$y1) && in_range($x2,$y2) && in_range($x3,$y3)) {
    ($x1,$y1) = transform($x1,$y1);
    ($dx,$dy) = transform($dx,$dy);
    printwrap("\\pic at ($x1,$y1) [transform shape] {my triangle={$dx,$dy}};");
  }
}

my @enc;
print "  \\begin{scope}[my grey]\n";
$Printwrap = 4;
foreach my $x ($enc_x_min .. $enc_x_max) {
  foreach my $y ($enc_y_min .. $enc_y_max) {
    foreach my $dir4 (0 .. 3) {
      my $dx = $dir4_to_dx[$dir4];
      my $dy = $dir4_to_dy[$dir4];

      my ($x2,$y2) = ($x+$dx, $y+$dy);
      ### triangle: "$x,$y $x2,$y2"
      my $n = $path->xyxy_to_n($x,$y, $x2,$y2);
      next unless defined $n && $n < $n_hi;

      if (Three_num_neighbours_of_xydxdy($x,$y, $dx,$dy) == 3) {
        push @enc, [$x,$y, $x2,$y2];
      } else {
        triangle($x,$y, $x2,$y2);
      }
    }
  }
}
printwrap_newline("\n");
print "  \\end{scope}\n";

$Printwrap = 2;
foreach my $elem (@enc) {
  triangle(@$elem);
}
printwrap_newline();

print "  \\coordinate (start) at (0,0);\n";
{
  my ($x,$y) = transform($x_hi,$y_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
}

{
  my ($x1,$y1) = transform($enc_x_min,$enc_y_min);
  my ($x2,$y2) = transform($enc_x_max,$enc_y_max);
  ($x1,$y1, $x2,$y2) = rect_corners_sort ($x1,$y1, $x2,$y2);
  my $xm = ($x1 + $x2) / 2;
  print "  \\coordinate (Three-minimum) at ($xm,$y1);\n";
}

print "  % end generated\n";
exit 0;
