#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::Geometry::Planar ();
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'transform_to_horizontal','xy_rotate_plus90','mul';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::CCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub transform {
  my ($x,$y) = @_;
  ($x,$y) = transform_to_horizontal($x,$y, $level+1);
  return ($x,$y);
}

my ($points,$ex,$ey) = MyCCurve::hull_points_and_end($level);
print "  \\node (start) at (0,0) [my start dot];\n";
{
  my ($x,$y) = transform($ex,$ey);
  print "  \\node (middle) at ($x,$y) [my start dot];\n";
}
{
  my ($x,$y) = mul($ex,$ey);
  ($x,$y) = transform($x,$y);
  print "  \\node (end) at ($x,$y) [my start dot];\n";
}
{
  my @names = ('P4','P3','P2','P1','P1dash','P2dash','P3dash','P4dash');
  $Printwrap = 2;
  foreach my $i (0 .. $#$points) {
    my $p = $points->[$i];
    my ($x,$y) = @$p;
    ($x,$y) = transform($x,$y);
    printwrap(" \\node ($names[$i]) at ($x,$y) [my dot];");
  }
  printwrap_newline();
  foreach my $i (0 .. $#$points) {
    my $p = $points->[$i];
    my ($x,$y) = @$p;
    print "% GP-Test  $names[$i]($level) == $x + I*$y\n";
  }
}
{
  my @names = ('P4a','P3a','P2a','P1a',
               'P1a-dash','P2a-dash','P3a-dash','P4a-dash');
  $Printwrap = 2;
  foreach my $i (0 .. $#$points) {
    my $p = $points->[$i];
    my ($x,$y) = @$p;
    ($x,$y) = xy_rotate_plus90($x,$y);
    $x += $ex; $y += $ey;
    ($x,$y) = transform($x,$y);
    printwrap(" \\node ($names[$i]) at ($x,$y) [my dot];");
  }
  printwrap_newline();
}

print "  % end generated\n";
exit 0;
