\\ Copyright 2014, 2015 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.

default(strictargs,1);
\\ allocatemem(300 * 10^6);


\\ Strichartz and Wang
\\ polroots(p)
\\ vecsort(apply(abs,polroots(p)))
\\ vecsort(polroots(p),(a,b)->a=abs(a);b=abs(b);cmp(b,a))
\\ root 1.954776399102463907266

sw_poly = x^9 - 3*x^8 + 3*x^7 - 3*x^6 + 2*x^5 + 4*x^4 - 8*x^3 + 8*x^2 - 16*x + 8;


\\-----------------------------------------------------------------------------
install(time, "lf");
elapsed_time__previous = time();
elapsed_time() =
{
  my(t = time(),
     delta = t - elapsed_time__previous);
  elapsed_time__previous = t;
  delta;
}

\\-----------------------------------------------------------------------------
str_poly_ascending__join_strings(v,sep) =
{
  concat(vector(2*#v-1,i, if(i%2,v[(i+1)\2],sep)));
}

str_poly_ascending(p) =
{
  if(type(p) == "t_VEC",
     return(str_poly_ascending__join_strings(apply(str_poly_ascending,p),
                                             "\n + ")));

  my(d = denominator(p));
  if(poldegree(d) != 0,
     p = numerator(p);
     my(low = polcoeff(d, 0));
     if(low==0, error("Oops, constant term of denominator is 0"));
     d /= low;
     p /= low;
     my(p_single = (poldegree(p) == 0
                    || poldegree(p) == valuation(p,variable(p))));
     my(d_single = (poldegree(d) == 0
                    || poldegree(d) == valuation(d,variable(d))));
     p_single = 0;
     d_single = 0;
     return(concat([if(p_single,"","("),
                    str_poly_ascending(p),
                    if(p_single,"",")"),
                    "/",
                    if(d_single,"","("),
                    str_poly_ascending(d),
                    if(d_single,"",")")
                   ])));
  if(poldegree(p) <= 0, return(Str(p)));
  my(str="");
  for(i=0,poldegree(p),
     my(coeff=polcoeff(p,i));
     if(coeff == 0, next());
     if(str!="",
        if(real(coeff)<0 || (real(coeff)==0 && imag(coeff)<0),
           str=concat(str," - "); coeff = -coeff,
           str=concat(str," + ")));
     str = concat([str,coeff*variable(p)^i]));
  str;
}

\\ str = str_poly_ascending( (x^2 + 1/2*x + 1)/(2*x^3 + x - 1) );
\\ print("str = "str);
\\ 
\\ str = str_poly_ascending( ((-1/4 + 1/4*I)*x^2 + (1/8 + 1/8*I))/(x^3 - 1/4*I*x + (1/8 + 1/8*I)) );
\\ print("str = "str);
\\ 
\\ str = str_poly_ascending( (x^2)/(2*x^3 + x - 1) );
\\ print("str = "str);

\\-----------------------------------------------------------------------------

cmp_by_abs(a,b) = cmp(abs(a),abs(b));
polroots_sorted(p) = vecsort(polroots(p),cmp_by_abs);

recurrence_check(r,v) =
{
  for(pos=1,#v-#r-1,
     my(n = sum(i=0,#r-1, v[pos+i]*r[i+1]));
     if(n!=v[pos+#r], return(0)));
  1;
}
recurrence_guess(v) =
{
  for(len=1,#v\2-4,
     my(m=matrix(len,len,i,j, v[i+j-1]),
        a=vectorv(len,i, v[i+len]));
     \\ print("len=" len);
     \\ print(m);
     \\ print(a);
     my(r = iferr(matsolve(m,a), e,0, errname(e)=="e_INV"));
     if(r==0, next());
     if(recurrence_check(r,v),
        print("len="len" recurrence ",r);
        my(p = Polrev(concat(r~,[-1])));
        print(p);
        \\ p = pol_partial_fractions(1/p);
        \\ for(i=1,#p, print("  ",str_poly_ascending(p[i])));
        my(f = factor(p*I)[,1]);
        for(i=1,#f,
           print("  ",if(f[i]==sw_poly,"sw_poly ",""),f[i]);
           my(rooots=polroots_sorted(f[i]));
           for(j=1,#rooots, print("    ",rooots[j],"   ",abs(rooots[j]))));
        return(0));
  );
}
recurrence_guess(vector(10,i,fibonacci(i)));

\\-----------------------------------------------------------------------------

conf_string(conf) = concat(apply(b->Str(b),conf));

conf_print(conf) =
{
  print("     .",if(conf[9],"--->","    "),".");
  print("    ",
        if(conf[7],"|"," "),
        if(conf[8],"^"," "),
        "    ",
        if(conf[10],"|"," "),
        if(conf[11],"^"," "));
  for(i=1,1, 
     print("    ",
           if(conf[7],"|"," "),
           if(conf[8],"|"," "),
           "    ",
           if(conf[10],"|"," "),
           if(conf[11],"|"," ")));
  print("    ",
        if(conf[7],"v"," "),
        if(conf[8],"|"," "),
        "    ",
        if(conf[10],"v"," "),
        if(conf[11],"|"," "));

  print(if(conf[6],"<---","    ")," .",
        if(conf[1],"<---","    "),". ",
        if(conf[12],"<---","    "),
        "  ",conf_string(conf),
        ", sides=",conf_num_sides(conf),
        if(conf_is_double_seg(conf),", double",""));
  print(if(conf[5],"--->","    "),"  ",
        if(conf[2],"--->","    "),"  ",
        if(conf[13],"--->","    "));

  print("    ",
        if(conf[4],"|"," "),
        if(conf[3],"^"," "),
        "    ",
        if(conf[15],"|"," "),
        if(conf[14],"^"," "));
  for(i=1,1, 
     print("    ",
           if(conf[4],"|"," "),
           if(conf[3],"|"," "),
           "    ",
           if(conf[15],"|"," "),
           if(conf[14],"|"," ")));
  print("    ",
        if(conf[4],"v"," "),
        if(conf[3],"|","."),
        "    ",
        if(conf[15],"v"," "),
        if(conf[14],"|","."));
  print("-----------------------------------------------");
}

conf_num_sides(conf) = \
  if(conf[1], 1 + conf[8] + conf[9] + conf[10],  0);
conf_is_double_seg(conf) = conf[1] && conf[2];

conf_print([1,1,1,1,1, 1,1,1,1,1, 1,1,1,1,1])

\\-----------------------------------------------------------------------------

\\           *---------*                      *---------*
\\          /|\  9|9  /|\                    /|\   |   /|\
\\         / | \  |  / | \                 7/ | \8 |10/ | \4
\\        /  |  \ | /  |  \                /  | 9\ | /9 |  \
\\       /  7|8  \ / 10|11 \              /   |   \ /   |   \
\\      *--- | ---*--  |  --*            *--- | ---*--  |  --*
\\     / \  7|8  / \ 10|11 / \          / \   |   / \   |   / \
\\    /   \  |  /   \  |  /   \       6/   \6 | 1/   \1 |12/   \12
\\   /  |  \ | /  |  \ | /  |  \      /  | 7\ | /8 |10\ | /4 |  \
\\  /  6|6  \|/  1|1  \|/ 12|12 \    /   |   \|/   |   \|/   |   \
\\ *---------*---------*---------*  *---------*---------*---------*
\\  \  5|5  /|\  2|2  /|\ 13|13 /    \   |   /|\   |   /|\   |   /
\\   \  |  / | \  |  / | \  |  /      \  | 4/ | \3 |15/ | \14|  /
\\    \   /  |  \   /  |  \   /       5\   /5 | 2\   /2 |13\   /13
\\     \ /  4|3  \ / 15|14 \ /          \ /   |   \ /   |   \ /
\\      *--  |  --*--  |  --*            *--  |  --*--  |  --*
\\       \  4|3  / \ 15|14 /              \   |   / \   |   /
\\        \  |  /   \  |  /                \  |  /   \  |  /
\\         \ | /     \ | /                 4\ | /3  15\ | /14
\\          \|/       \|/                    \|/       \|/
\\           *         *                      *         *
\\
\\ 6 -> left 2bit 2,9
\\ 

\\ expanded    1  2  3 4 5   6  7 8   9 19 11  12,13,14,15
left_perm  = [15, 1,10,3,2,  9, 8,5, 10, 1, 4,  8, 9, 6, 7];
right_perm = [ 3, 2,15,8,1, 13,10,9, 14, 1, 8,  4,12, 9,10];

perms_vector = [left_perm,right_perm];
conf_permute(conf,perm) = vector(#conf,i, conf[perm[i]+1]);

\\---------

\\ c=vector(15);
\\ c[6+1] = 1;
\\ c[9+1] = 1;
\\ c = conf_permute(c,left_perm);
\\ c
\\ conf_print(c)
\\ quit

conf_initial_vector = vector(15,c, vector(15,i, i==c));
\\ conf_initial[6+1] = 1;
\\ conf_initial
\\ print(conf_string(conf_initial));
for(i=1,#conf_initial_vector, conf_print(conf_initial_vector[i]));

\\ conf_permute(conf_initial,left_perm)
\\ conf_permute(conf_initial,right_perm)
\\ conf_print(conf_permute(conf_initial,left_perm));
\\ conf_print(conf_permute(conf_initial,right_perm));

\\-------------------------------------

conf_to_index_key   = List([]);
conf_to_index_value = List([]);
conf_to_index_num   = 0;
index_to_conf = List([]);
conf_to_index(conf) =
{
  my(pos=setsearch(conf_to_index_key,conf));
  if(pos>0, return(conf_to_index_value[pos]));
  pos = setsearch(conf_to_index_key,conf,1);
  conf_to_index_num++;
  listput(index_to_conf,conf);
  listinsert(conf_to_index_key,conf, pos);
  listinsert(conf_to_index_value,conf_to_index_num, pos);

  conf = conf_permute(conf,mirror_perm);
  pos = setsearch(conf_to_index_key,conf,1);
  if(pos,
     listinsert(conf_to_index_key,conf, pos);
     listinsert(conf_to_index_value,conf_to_index_num, pos));

  conf_to_index_num;
}

\\ \\ list is a sorted list.
\\ \\ Insert value into it, if not already present.
\\ listinsert_sorted(list,value) =
\\ {
\\   my(pos = setsearch(list,value,1));
\\   print(pos);
\\   if(pos, listinsert(list,value,pos));
\\   print(list);
\\ }
\\ {
\\   my(l=List([]));
\\   listinsert_sorted(l,3);
\\   print(l);
\\   Vec(l)==[3] || error("oops");
\\ }
\\ quit

\\ vector(#conf_initial_vector,i, conf_to_index(conf_initial_vector[i]));

size = 754;
size = 394;  \\ merge mirror images
print("size=",size);
m = matrix(size,size);
\\ m = matrix(103,103);
m_lr=matrix(size,2);
{
my(pending=vector(2*size),
   pending_last = 0);
for(i=1,#conf_initial_vector,
   pending_last++;
   pending[pending_last] = conf_initial_vector[i]);
my(seen=vector(size));
while(pending_last > 0,
      \\ print("pending ",pending);
      my(conf       = pending[pending_last],
         conf_index = conf_to_index(conf));
      pending_last--;
      \\ print("conf ",conf_string(conf),"  ",conf_index);

      if(seen[conf_index],
         print("  skip seen");
         next());
      seen[conf_index] = 1;

      for(p=1,#perms_vector,
         my(new_conf       = conf_permute(conf,perms_vector[p]),
            new_conf_index = conf_to_index(new_conf));
         print("  new conf ",conf_string(new_conf),"  ",new_conf_index);
         m[new_conf_index,conf_index]++;
         m_lr[conf_index,p] = new_conf_index;
         pending_last++;
         pending[pending_last] = new_conf;
      ));
}

print("total configurations ",conf_to_index_num);
print("determinant ",matdet(m));
size = matsize(m)[1];
if(conf_to_index_num != size, error("oops, num!=size"));

print("total weight ", sum(i=1,size,sum(j=1,size,m[i,j])));
initial = vectorv(size,i, i==1);

initial = vectorv(size,i, 0);
for(i=1,#conf_initial_vector, \
    initial[conf_to_index(conf_initial_vector[i])]++);
\\ print("initial "initial)

\\----------------------
\\ print conf descents

\\ {
\\ my(v=initial);
\\ for(k=0,5,
\\    print("confs k=" k);
\\    for(i=1,#v, if(v[i], conf_print(index_to_conf[i])));
\\    for(i=1,#v, if(v[i],
\\                   my(conf=index_to_conf[i]);
\\                   print(conf_string(conf),
\\                         " -> ",
\\                         conf_string(conf_permute(conf,left_perm)),
\\                         " "
\\                         conf_string(conf_permute(conf,right_perm)))));
\\    v = m*v);
\\ }

\\-----------------------------------------------------------------------------

{
  my(count=0);
  for(i=1,size,
     for(j=1,size,
        if(m[i,j]>1,
           print(i","j" m="m[i,j]);
           count++)));
  print("total " count " doubles");
}
{
  my(unreached=vector(size,i,1));
  for(i=1,size, for(j=1,2, unreached[m_lr[i,j]]=0));
  for(i=1,#unreached,
     if(unreached[i],
        print("unreached ",i,"  ",conf_string(index_to_conf[i]));
        conf_print(index_to_conf[i])));
  print("total unreached ",vecsum(unreached));
}
\\ {
\\   my(unseen=vector(size,i,1));
\\   for(i=1,size,
\\      my(p=m_lr[i,1]);
\\      if(!unseen[p], m_lr[i,1]=m_lr[i,2]; m_lr[i,2]=p); \\ swap
\\      unseen[m_lr[i,1]]=0);
\\   print("unseen ",vecsum(unseen));
\\ }
\\ {
\\   my(unseen=vector(size,i,1));
\\   for(i=1,size, unseen[m_lr[i,1]]=0; unseen[m_lr[i,2]]=0);
\\   print("unseen ",vecsum(unseen));
\\ }

\\-----------------------------------------------------------------------------

\\ works, but the same speed
mul(a) =
{
  my(ret=matrix(size,size));
  for(i=1,size,
     for(j=1,#m_lr,
        my(r = m_lr[i,j]);
        ret[r,] = ret[r,] + a[i,];
        \\ ret[r,] += a[i,];        \\ no good for gp2c
        ));
  ret;
}

\\ print("mul");
\\ my(t=mul(m)); t==m^2
\\ print("  took ",elapsed_time());
\\ print("square");
\\ m^2;print1("");
\\ print("  took ",elapsed_time());

\\-----------------------------------------------------------------------------
s_matrix(n) = my(m=matrix(size,size)); m[1,n]=1; m;

\\ C*s shifts row 1 to row n, all others 0
column_1_to_n(n) = my(m=matrix(size,size)); m[n,1]=1; m;

\\  1 0 0 0    0 1 0 0    0 0 1 0    0 0 0 1 
\\  0 0 0 0    0 0 0 0    0 0 0 0    0 0 0 0 
\\  0 0 0 0    0 0 0 0    0 0 0 0    0 0 0 0 
\\  0 0 0 0    0 0 0 0    0 0 0 0    0 0 0 0 
\\
\\  0 0 0 0    0 0 0 0    0 0 0 0    0 0 0 0 
\\  1 0 0 0    0 1 0 0    0 0 1 0    0 0 0 1 
\\  0 0 0 0    0 0 0 0    0 0 0 0    0 0 0 0 
\\  0 0 0 0    0 0 0 0    0 0 0 0    0 0 0 0 
\\
\\  0 0 0 0
\\  1 0 0 0
\\  0 0 0 0
\\  0 0 0 0

\\ row 1 from each power
\\ T = S0*m^0 + S1*m^1 + S2*m^2 + S3*m^3
\\ C*T * m * (C*T)^-1

\\-----------------------------------------------------------------------------

\\ print("matsolve\n");
\\ mx = matid(size) - m*x;
\\ gm = matsolve(mx,initial);
\\ gm
